;
;  dosio.asm
;  Support for BIOS calls in NCSA PCShow
;****************************************************************************
;*																			*
;*																			*
;*	  part of NCSA PCShow												 	*
;*	  by Quincey Koziol													 	*
;*																			*
;*	  National Center for Supercomputing Applications						*
;*	  152 Computing Applications Building									*
;*	  605 E. Springfield Ave.												*
;*	  Champaign, IL  61820													*
;*																		 	*
;*																		 	*
;****************************************************************************

	TITLE	DOSIO	-- LOW-LEVEL I/O FOR SANE HARDWARE HANDLING

;
;	From original code by Tim Krauskopf	1984-1985
;	National Center for Supercomputing Applications
;
	NAME	DOSIO

;
;  Macros for reading and writing I/O ports
;
MOUT	MACRO	REG,STUFF		; one byte to the given I/O register
	MOV	DX,REG
	MOV	AL,STUFF
	OUT	DX,AL
	ENDM
;
MIN	MACRO	REG		 	; get one byte to al
	MOV	DX,REG
	IN	AL,DX
	ENDM
;
;  Internal data 
;
	X EQU	6				;  for the large model programs

	INCLUDE	DOS.MAC
	DSEG

	PUBLIC	DTAPTR				; POINTER TO DTA LOCATION

DTAPTR	DW	0000H				; dta ADDRESS FOR ME
DTADS	DW	0000H				; ds FOR dta

	ENDDS
;
;
;
;	The subroutines to call from C
;
	PSEG

	PUBLIC  FIND_1ST,FINDNEXT,SETDIR,GETDIR


;**********************************************************************
;
;  find first
;	make dos find file names according to wildcards
;  findfirst(filename,attr)
;	char *filename; int attr;
;
FIND_1ST	PROC	FAR
	PUSH	BP
	MOV		BP,SP
	PUSH	ES
	PUSH	DS
	
	MOV		AH,02FH					; dos FUNCTION GET dta
	INT		21H
	MOV		DTAPTR,BX				; SQUIRREL A COPY FOR ME
	MOV		AX,ES
	MOV		DTADS,AX

	MOV		AX,[BP+X+2]				; DS OF FILENAME PTR
	MOV		DS,AX
	MOV		DX,[BP+X]				; PTR PART OF FILENAME
	MOV		CX,[BP+X+4]				; ATTRIBUTE TO SEARCH FOR
	MOV		AH,04EH					; FIND MATCHING FILE dos CALL
	INT		21H
	JC		BADRET					; AX ALREADY CONTAINS ERROR CODE
	XOR		AX,AX

BADRET:
	POP		DS
	POP		ES
	POP		BP
	RET
FIND_1ST	ENDP

;
;  findnext()
;  will find entries that follow findfirst
;  no need to respecify file name
;
FINDNEXT	PROC	FAR
	PUSH	BP

	MOV		AH,04FH					; FIND NEXT dos CALL
	INT		21H
	JC		NBADRET
	XOR		AX,AX

NBADRET:
	POP		BP
	RET
FINDNEXT	ENDP

;
; setdir(dir_name);
;	unsigned char *dir_name;
;	sets the current directory to dir_name
;
SETDIR	PROC	FAR
	PUSH	BP
	MOV		BP,SP
	PUSH	DS

	MOV		AX,[BP+X+2]				; GET THE SEGMENT OF THE DIRECTORY
	MOV		DS,AX
	MOV		DX,[BP+X]				; GET THE OFFSET OF THE DIRECTORY
	MOV		AH,03BH					; SET DIRECTORY dos	CALL
	INT		21H
	JC		SBADRET
	XOR		AX,AX

SBADRET:
	POP		DS
	POP		BP
	RET
SETDIR		ENDP
;
;	getdir(path_name)
;	unsigned char *path_name
;	get the full path including drive from dos and places it in path_name
;
GETDIR	PROC	FAR
	PUSH	BP
	MOV		BP,SP
	PUSH	DS

	MOV		AX,[BP+X+2]				; GET THE SEGMENT OF THE STRING
	MOV		DS,AX
	MOV		SI,[BP+X]				; GET THE OFFSET OF THE STRING
	MOV		AH,019H					; dos CALL TO GET THE CURRENT DRIVE
	INT		21H						; CALL THE DOS INTERUPT
	ADD		AL,65					; CONVERT THE DRIVE NUMBER RETURNED INTO ASCII
	MOV		BYTE PTR DS:[SI],AL		; SAVE THE DRIVE NUMBER
	INC		SI						; MOVE TO THE NEXT CHARACTER
	MOV		BYTE PTR DS:[SI],':'	; PUT A ':' AFTER THE DRIVE NUMBER
	INC		SI						; MOVE TO THE NEXT CHARACTER
	MOV		BYTE PTR DS:[SI],'\'	; PUT A BACKSLASH BEFORE THE PATH NAMES
	INC		SI						; INCREMENT TO THE SPACE FOR THE PATH NAME
	MOV		DL,0					; DL=0 MEANS DEFAULT DRIVE TO GET THE PATH FROM
	MOV		AH,047H					; dos CALL TO GET THE CURRENT PATH
	INT		21H						; GET THE PATH FROM dos
	JC		GBADRET					; CHECK FOR ERROR RETURNED
	XOR		AX,AX					; ZERO ACC FOR NO ERROR

GBADRET:
	POP		DS
	POP		BP
	RET
GETDIR		ENDP

	ENDPS
	END
